@@ -32,6 +32,7 @@
 #include <QFont>
 #include <QHash>
 #include <QIcon>
+#include <QMimeData>
 
 #include <gpgme++/key.h>
 
@@ -73,6 +74,8 @@ public:
     bool m_modelResetInProgress = false;
     KeyList::Options m_keyListOptions = AllKeys;
     std::vector<GpgME::Key> m_remarkKeys;
+    std::shared_ptr<DragHandler> m_dragHandler;
+    std::vector<Key::Origin> extraOrigins;
 };
 
 AbstractKeyListModel::Private::Private(Kleo::AbstractKeyListModel *qq)
@@ -223,7 +226,7 @@ QModelIndex AbstractKeyListModel::index(const KeyGroup &group, int col) const
     }
 }
 
-void AbstractKeyListModel::setKeys(const std::vector<Key> &keys)
+void AbstractKeyListModel::setKeys(const std::vector<Key> &keys, const std::vector<Key::Origin> &extraOrigins)
 {
     const bool inReset = modelResetInProgress();
     if (!inReset) {
@@ -231,6 +234,7 @@ void AbstractKeyListModel::setKeys(const std::vector<Key> &keys)
     }
     clear(Keys);
     addKeys(keys);
+    d->extraOrigins = extraOrigins;
     if (!inReset) {
         endResetModel();
     }
@@ -318,35 +322,39 @@ QVariant AbstractKeyListModel::headerData(int section, Qt::Orientation o, int ro
         if (role == Qt::DisplayRole || role == Qt::EditRole || role == Qt::ToolTipRole) {
             switch (section) {
             case PrettyName:
-                return i18n("Name");
+                return i18nc("@title:column", "Name");
             case PrettyEMail:
-                return i18n("E-Mail");
+                return i18nc("@title:column", "E-Mail");
             case Validity:
-                return i18n("User-IDs");
+                return i18nc("@title:column", "User-IDs");
             case ValidFrom:
-                return i18n("Valid From");
+                return i18nc("@title:column", "Valid From");
             case ValidUntil:
-                return i18n("Valid Until");
+                return i18nc("@title:column", "Valid Until");
             case TechnicalDetails:
-                return i18n("Protocol");
+                return i18nc("@title:column", "Protocol");
             case ShortKeyID:
-                return i18n("Key-ID");
+                return i18nc("@title:column", "Key-ID");
             case KeyID:
-                return i18n("Key-ID");
+                return i18nc("@title:column", "Key-ID");
             case Fingerprint:
-                return i18n("Fingerprint");
+                return i18nc("@title:column", "Fingerprint");
             case Issuer:
-                return i18n("Issuer");
+                return i18nc("@title:column", "Issuer");
             case SerialNumber:
-                return i18n("Serial Number");
+                return i18nc("@title:column", "Serial Number");
             case Origin:
-                return i18n("Origin");
+                return i18nc("@title:column", "Origin");
             case LastUpdate:
-                return i18n("Last Update");
+                return i18nc("@title:column", "Last Update");
             case OwnerTrust:
-                return i18n("Certification Trust");
+                return i18nc("@title:column", "Certification Trust");
             case Remarks:
-                return i18n("Tags");
+                return i18nc("@title:column", "Tags");
+            case Algorithm:
+                return i18nc("@title:column", "Algorithm");
+            case Keygrip:
+                return i18nc("@title:column", "Keygrip");
             case NumColumns:;
             }
         }
@@ -376,7 +384,7 @@ QVariant AbstractKeyListModel::data(const QModelIndex &index, int role) const
 {
     const Key key = this->key(index);
     if (!key.isNull()) {
-        return data(key, index.column(), role);
+        return data(key, index.row(), index.column(), role);
     }
 
     const KeyGroup group = this->group(index);
@@ -387,7 +395,7 @@ QVariant AbstractKeyListModel::data(const QModelIndex &index, int role) const
     return QVariant();
 }
 
-QVariant AbstractKeyListModel::data(const Key &key, int column, int role) const
+QVariant AbstractKeyListModel::data(const Key &key, int row, int column, int role) const
 {
     if (role == Qt::DisplayRole || role == Qt::EditRole || role == Qt::AccessibleTextRole) {
         switch (column) {
@@ -448,6 +456,9 @@ QVariant AbstractKeyListModel::data(const Key &key, int column, int role) const
         case Issuer:
             return QString::fromUtf8(key.issuerName());
         case Origin:
+            if (key.origin() == Key::OriginUnknown && (int)d->extraOrigins.size() > row) {
+                return Formatting::origin(d->extraOrigins[row]);
+            }
             return Formatting::origin(key.origin());
         case LastUpdate:
             if (role == Qt::AccessibleTextRole) {
@@ -489,6 +500,14 @@ QVariant AbstractKeyListModel::data(const Key &key, int column, int role) const
             }
         }
             return QVariant();
+        case Algorithm:
+            return Formatting::prettyAlgorithmName(key.subkey(0).algoName());
+        case Keygrip:
+            if (role == Qt::AccessibleTextRole) {
+                return Formatting::accessibleHexID(key.subkey(0).keyGrip());
+            } else {
+                return Formatting::prettyID(key.subkey(0).keyGrip());
+            }
         case NumColumns:
             break;
         }
@@ -552,7 +571,15 @@ QVariant AbstractKeyListModel::data(const KeyGroup &group, int column, int role)
     } else if (role == Qt::FontRole) {
         return QFont();
     } else if (role == Qt::DecorationRole) {
-        return column == Icon ? QIcon::fromTheme(QStringLiteral("group")) : QVariant();
+        if (column != Icon && column != Summary) {
+            return QVariant();
+        }
+        return Kleo::all_of(group.keys(),
+                            [](const auto &key) {
+                                return key.hasEncrypt();
+                            })
+            ? QIcon::fromTheme(QStringLiteral("group"))
+            : QIcon::fromTheme(QStringLiteral("emblem-warning"));
     } else if (role == Qt::BackgroundRole) {
     } else if (role == Qt::ForegroundRole) {
     } else if (role == GroupRole) {
@@ -1579,6 +1606,38 @@ AbstractKeyListModel *AbstractKeyListModel::createHierarchicalKeyListModel(QObje
     return m;
 }
 
+QMimeData *AbstractKeyListModel::mimeData(const QModelIndexList &indexes) const
+{
+    if (d->m_dragHandler) {
+        return d->m_dragHandler->mimeData(indexes);
+    } else {
+        return QAbstractItemModel::mimeData(indexes);
+    }
+}
+
+Qt::ItemFlags AbstractKeyListModel::flags(const QModelIndex &index) const
+{
+    if (d->m_dragHandler) {
+        return d->m_dragHandler->flags(index);
+    } else {
+        return QAbstractItemModel::flags(index);
+    }
+}
+
+QStringList AbstractKeyListModel::mimeTypes() const
+{
+    if (d->m_dragHandler) {
+        return d->m_dragHandler->mimeTypes();
+    } else {
+        return QAbstractItemModel::mimeTypes();
+    }
+}
+
+void AbstractKeyListModel::setDragHandler(const std::shared_ptr<DragHandler> &dragHandler)
+{
+    d->m_dragHandler = dragHandler;
+}
+
 #include "keylistmodel.moc"
 
 /*!
