@@ -23,7 +23,6 @@
 
 #include <KLocalizedString>
 
-#include <QInputDialog>
 #include <QListView>
 #include <QMenu>
 #include <QPointer>
@@ -410,3 +409,5 @@ void DirectoryServicesWidget::clear()
 }
 
 #include "directoryserviceswidget.moc"
+
+#include "moc_directoryserviceswidget.cpp"
