@@ -33,7 +33,7 @@
 
 bool Kleo::DeVSCompliance::isActive()
 {
-    return getCryptoConfigStringValue("gpg", "compliance") == QLatin1String{"de-vs"};
+    return getCryptoConfigStringValue("gpg", "compliance") == QLatin1StringView{"de-vs"};
 }
 
 bool Kleo::DeVSCompliance::isCompliant()
@@ -77,6 +77,17 @@ bool Kleo::DeVSCompliance::allSubkeysAreCompliant(const GpgME::Key &key)
     });
 }
 
+bool Kleo::DeVSCompliance::userIDIsCompliant(const GpgME::UserID &id)
+{
+    if (!isActive()) {
+        return true;
+    }
+    return (id.parent().keyListMode() & GpgME::Validate) //
+        && !id.isRevoked() //
+        && id.validity() >= GpgME::UserID::Full //
+        && allSubkeysAreCompliant(id.parent());
+}
+
 bool Kleo::DeVSCompliance::keyIsCompliant(const GpgME::Key &key)
 {
     if (!isActive()) {
