@@ -9,12 +9,12 @@
 
 #pragma once
 
-#include <QObject>
+#include "kleo_export.h"
 
-#include <kleo_export.h>
+#include <QObject>
+#include <QStringList>
 
 class QString;
-#include <QStringList>
 
 namespace Kleo
 {
@@ -25,7 +25,7 @@ class KLEO_EXPORT FileSystemWatcher : public QObject
 public:
     explicit FileSystemWatcher(QObject *parent = nullptr);
     explicit FileSystemWatcher(const QStringList &paths, QObject *parent = nullptr);
-    ~FileSystemWatcher();
+    ~FileSystemWatcher() override;
 
     void setDelay(int ms);
     int delay() const;
@@ -53,4 +53,3 @@ private:
     QScopedPointer<Private> const d;
 };
 }
-
